#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Mirror images creator
# version-1.0 Aug 2021
# Copyright (c) 2021 TerraX
# inspired by and based on the high-pass-sharpen.py by Paul Sherman
#
# Impliments a live work on active image so that the plugin
# does not need to be restarted when testing the effect.
# This plugin has been tested on GIMP 2.10.18
# --------------------------------------------------------------------
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ---------------------------------------------------------------------
import gimp, gimpplugin, math
from gimpenums import *
pdb = gimp.pdb
import gtk, gimpui, gimpcolor
from gimpshelf import shelf


class reuse_init(object):
	ausgabe = 0
	dx = 0
	drot = 0
	dflip = 0
	reset = 0
	original = None

	def make_label(self, text):
		label = gtk.Label(text)
		label.set_use_underline(True)
		label.set_alignment(0.5, 0.5)
		label.show()
		return label

	def make_slider_and_spinner(self, init, minval, maxval, step, page, digits):				#erzeugt die Schieberegler
		controls = {'adj': gtk.Adjustment(init, minval, maxval, step, page), 'slider': gtk.HScale(), 'spinner': gtk.SpinButton()}
		controls['slider'].set_adjustment(controls['adj'])
		controls['slider'].set_draw_value(False)
		controls['spinner'].set_adjustment(controls['adj'])
		controls['spinner'].set_digits(digits)
		controls['slider'].show()
		controls['spinner'].show()
		return controls

	def show_error_msg(self, msg):
		origMsgHandler = pdb.gimp_message_get_handler()
		pdb.gimp_message_set_handler(ERROR_CONSOLE)
		pdb.gimp_message(msg)
		pdb.gimp_message_set_handler(origMsgHandler)

	def stringToColor(self, string):
		colorlist = string[5:-1].split(", ")
		return gimpcolor.RGB(float(colorlist[0]), float(colorlist[1]), float(colorlist[2]), float(colorlist[3]))


class mirror_img(reuse_init):
	def __init__(self, runmode, img, drawable):
		self.img = img													#ohne gehts nicht
		self.drawable = drawable
		self.original = img.layers[0]
		self.ausgabe = self.create(self.img)							#erzeugt die notwendigen Ebenen
		self.showDialog(img)

	def showDialog(self, img):
		self.image = img												#ohne gehts nicht
		self.dialog = gimpui.Dialog("Spiegelbilder", "SpiegelBilderDialog")

		self.table = gtk.Table(6, 6, True)
		self.table.set_homogeneous(False)
		self.table.set_row_spacings(8)
		self.table.set_col_spacings(8)
		self.table.show()

		b = int(pdb.gimp_image_width(img) / 4)
		self.distance_label_shift = self.make_label("x-Shift:")			#Erzeugung des Schiebereglers für den Versatz
		self.table.attach(self.distance_label_shift, 0, 1, 1, 2)
		self.shift_adj = self.make_slider_and_spinner(0, -b, b, 1, 10, 0)						#min und max werden auf +/- die halbe Original Bildbreite gesetzt
		self.distance_label_shift.set_mnemonic_widget(self.shift_adj['spinner'])
		self.table.attach(self.shift_adj['slider'], 1, 6, 1, 2)
		self.table.attach(self.shift_adj['spinner'], 6, 7, 1, 2)
		s_id = self.shift_adj['adj'].connect("value_changed", self.xshift, img)					#Anbindung des auszuführenden Codes, wenn der Schieberegler verändert wird.
																								#Funktioniert nur als Funktion, nicht als Prozedur

		self.distance_label_rot = self.make_label("Rotation:")			#Erzeugung des Schiebereglers für die Rotation
		self.table.attach(self.distance_label_rot, 0, 1, 4, 5)
		self.rot_adj = self.make_slider_and_spinner(0, -90, 90, 1, 10, 0)						#min und max werden auf +/- 90° gesetzt
		self.distance_label_rot.set_mnemonic_widget(self.rot_adj['spinner'])
		self.table.attach(self.rot_adj['slider'], 1, 6, 4, 5)
		self.table.attach(self.rot_adj['spinner'], 6, 7, 4, 5)
		r_id = self.rot_adj['adj'].connect("value_changed", self.rotation, img)					#Anbindung des auszuführenden Codes, wenn der Schieberegler verändert wird.
																								#Funktioniert nur als Funktion, nicht als Prozedur

		self.flip_button = gtk.Button("Flip Seiten")					#Erzeugung des Resetbuttons
		self.flip_button.connect("clicked", self.flip, img)				#Anbindung des auszuführenden Codes, wenn der Button geklickt wird
		self.flip_button.set_size_request(110, -1)
		self.flip_button.show()

		self.dialog.vbox.hbox1 = gtk.HBox(False, 7)
		self.dialog.vbox.hbox1.show()
		self.dialog.vbox.pack_start(self.dialog.vbox.hbox1, True, True, 7)
		self.dialog.vbox.hbox1.pack_start(self.table, True, True, 7)

		reset_button = gtk.Button("_Reset")								#Erzeugung des Resetbuttons
		reset_button.connect("clicked", self.resetbutton)				#Anbindung des auszuführenden Codes, wenn der Button geklickt wird
		reset_button.show()

		if gtk.alternative_dialog_button_order():						#Anordnung der Buttons
			ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
			cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
			self.dialog.action_area.add(reset_button)
			self.dialog.action_area.add(self.flip_button)
		else:
			self.dialog.action_area.add(self.flip_button)
			self.dialog.action_area.add(reset_button)
			cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
			ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
		ok_button.connect("clicked", self.okbutton)
		cancel_button.connect("clicked", self.cancelbutton)

		self.dialog.show()
		self.dialog.run()

	def cancelbutton(self, widget):										#widget, auch wenns ungenutzt ist, muss mit übertragen werden
		image = self.img
		self.loeschen(image, "Links")									#löschen der erzeugten Ebenengruppen
		self.loeschen(image, "Rechts")
		pdb.plug_in_autocrop(image, self.original)						#Originalbild wieder herstellen
		pdb.gimp_layer_set_visible(self.original, TRUE)
		gimp.displays_flush()  											# Bildaktualisierung

	def okbutton(self, widget):											#widget, auch wenns ungenutzt ist, muss mit übertragen werden
		image = self.img
		width = pdb.gimp_image_width(image)
		height = pdb.gimp_image_height(image)
		Kopie = pdb.gimp_layer_copy(self.original, FALSE)				#neue Ebene aus dem Originalbild mit der eingestellten Verschiebung und Drehung erzeugen
		pdb.gimp_image_insert_layer(image, Kopie, None, 0)
		if self.dflip == 1:
			Kopie = pdb.gimp_item_transform_flip(Kopie, width / 2, 0, width / 2, height)
		Kopie = pdb.gimp_item_transform_rotate(Kopie, math.radians(self.drot), TRUE, 0, 0)
		pdb.gimp_layer_translate(Kopie, self.dx, 0)
		pdb.gimp_image_select_rectangle(image, CHANNEL_OP_REPLACE, width / 2, 0, width, height)
		Auswahl = pdb.gimp_edit_cut(Kopie)
		pdb.gimp_image_select_rectangle(image, CHANNEL_OP_REPLACE, 0, 0, width / 2, height)
		Auswahl = pdb.gimp_edit_copy(Kopie)
		floating_sel = pdb.gimp_edit_paste(Kopie, TRUE)
		floating_sel = pdb.gimp_item_transform_flip(floating_sel, width / 2, 0, width / 2, height)
		pdb.gimp_floating_sel_anchor(floating_sel)
		pdb.gimp_layer_set_visible(Kopie, TRUE)
		self.loeschen(image, "Links")									#löschen der erzeugten Ebenengruppen
		self.loeschen(image, "Rechts")
		pdb.plug_in_autocrop_layer(image, Kopie)  						# Bild auf genutzte Fläche beschneiden
		gimp.displays_flush()  											# Bildaktualisierung

	def loeschen(self, image, name):
		pdb.gimp_selection_none(image)
		layer = pdb.gimp_image_get_layer_by_name(image, name)
		pdb.gimp_image_remove_layer(image, layer)

	def resetbutton(self, widget):
		image = self.img
		self.loeschen(image, "Links")									#löschen der erzeugten Ebenengruppen
		self.loeschen(image, "Rechts")
		pdb.plug_in_autocrop(image, self.original)						#Originalbild wieder herstellen
		self.ausgabe = self.create(image)								#Ebenengruppen neu erzeugen
		self.reset = 1													#Rücksetzen der Adjustments, ohne die "value_changed" Aktion auszuführen
		self.shift_adj['adj'].set_value(0)
		self.dx = 0
		self.rot_adj['adj'].set_value(0)
		self.drot = 0
		self.reset = 0
		gimp.displays_flush()											#Bildaktualisierung

	def flip(self, widget, image):										#widget, auch wenns ungenutzt ist, muss mit übertragen werden
		width = pdb.gimp_image_width(image)
		height = pdb.gimp_image_height(image)
		KopieA = pdb.gimp_image_get_layer_by_name(image, "KopieL")
		KopieA = pdb.gimp_item_transform_flip(KopieA, width / 2, 0, width / 2, height)
		KopieB = pdb.gimp_image_get_layer_by_name(image, "KopieR")
		KopieB = pdb.gimp_item_transform_flip(KopieB, width / 2, 0, width / 2, height)
		if self.dflip == 0:
			self.dflip = 1
		else:
			self.dflip = 0
		self.reset = 1													#Spiegeln der Adjustments, ohne die "value_changed" Aktion auszuführen
		self.dx = -1 * self.dx
		self.shift_adj['adj'].set_value(self.dx)
		self.drot = -1 * self.drot
		self.rot_adj['adj'].set_value(self.drot)
		self.reset = 0
		gimp.displays_flush()											#Bildaktualisierung

	def rotation(self, widget, image):									#widget, auch wenns ungenutzt ist, muss mit übertragen werden
		if self.reset == 0:
			ang_rad = self.rot_adj['adj'].get_value()
			ang = math.radians(ang_rad - self.drot)						#Relative Verschiebung zur letzten Aktion bestimmen und in Bogenmaß umrechnen
			KopieA = pdb.gimp_image_get_layer_by_name(image, "KopieL")
			KopieB = pdb.gimp_image_get_layer_by_name(image, "KopieR")
			KopieA = pdb.gimp_item_transform_rotate(KopieA, ang, TRUE, 0, 0)		#Rotation ausführen
			KopieB = pdb.gimp_item_transform_rotate(KopieB, -ang, TRUE, 0, 0)
			self.drot = ang_rad											#aktuelle Rotation in Grad speichern. Erzeugt weniger Rundungsfehler als in rad.
			gimp.displays_flush()
		return 1

	def xshift(self, widget, image):									#widget, auch wenns ungenutzt ist, muss mit übertragen werden
		if self.reset == 0:
			wert = self.shift_adj['adj'].get_value()
			s = wert - self.dx											#relative Verschiebung zurm letzten Aktion
			KopieA = pdb.gimp_image_get_layer_by_name(image, "KopieL")
			KopieB = pdb.gimp_image_get_layer_by_name(image, "KopieR")
			pdb.gimp_layer_translate(KopieA, s, 0)						#Verschiebung ausführen
			pdb.gimp_layer_translate(KopieB, -s, 0)
			self.dx = wert												#aktuelle Verschiebung speichern
			gimp.displays_flush()

	def create(self, image):

		pdb.gimp_context_set_background((255, 255, 255))  # weiss
		pdb.gimp_context_set_foreground((0, 0, 0))  # schwarz

		original = self.original
		width = pdb.gimp_image_width(image)
		height = pdb.gimp_image_height(image)

		GruppeA = pdb.gimp_layer_group_new(image)						#die linke Bildhälfte
		GruppeA.name = "Links"
		pdb.gimp_image_insert_layer(image, GruppeA, None, 0)
		KopieA = pdb.gimp_layer_copy(original, FALSE)
		KopieA.name = "KopieL"
		pdb.gimp_image_insert_layer(image, KopieA, GruppeA, 0)
		pdb.gimp_layer_set_mode(KopieA, LAYER_MODE_DARKEN_ONLY_LEGACY)
		pdb.gimp_layer_set_visible(KopieA, TRUE)

		GruppeB = pdb.gimp_layer_group_new(image)						#die rechte Bildhälfte
		GruppeB.name = "Rechts"
		pdb.gimp_image_insert_layer(image, GruppeB, None, 0)
		KopieB = pdb.gimp_layer_copy(original, FALSE)
		KopieB.name = "KopieR"
		pdb.gimp_image_insert_layer(image, KopieB, GruppeB, 0)
		pdb.gimp_layer_set_mode(KopieB, LAYER_MODE_DARKEN_ONLY_LEGACY)
		pdb.gimp_layer_set_visible(KopieB, TRUE)

		pdb.gimp_image_select_rectangle(image, CHANNEL_OP_REPLACE, 0, 0, width, height)
		pdb.gimp_image_set_active_layer(image, KopieB)
		KopieB = pdb.gimp_item_transform_flip(KopieB, width / 2, 0, width / 2, height)
		pdb.gimp_floating_sel_anchor(KopieB)
		nheight = 1.2 * height
		pdb.gimp_image_resize(image, 2 * width, nheight, width / 2, height * 0.1)			#Bildfläche vergrößern, damit bei der Drehung nichts abgeschnitten wird und neu zentrieren.

		weiss_l = pdb.gimp_layer_new(image, 2 * width, nheight, RGBA_IMAGE, "weiss-links", 100, LAYER_MODE_NORMAL_LEGACY)
		pdb.gimp_drawable_fill(weiss_l, FILL_WHITE)
		pdb.gimp_image_insert_layer(image, weiss_l, GruppeA, 1)

		weiss_r = pdb.gimp_layer_new(image, 2 * width, nheight, RGBA_IMAGE, "weiss-rechts", 100, LAYER_MODE_NORMAL_LEGACY)
		pdb.gimp_drawable_fill(weiss_r, FILL_WHITE)
		pdb.gimp_image_insert_layer(image, weiss_r, GruppeB, 1)

		# mit den Masken wird die jeweils uninteressante Bildhälfte ausgeblendet, ohne sie zu löschen.
		maskws = pdb.gimp_layer_create_mask(weiss_l, ADD_MASK_BLACK)
		masksw = pdb.gimp_layer_create_mask(weiss_r, ADD_MASK_WHITE)

		pdb.gimp_layer_add_mask(weiss_l, maskws)
		pdb.gimp_layer_add_mask(weiss_r, masksw)

		pdb.gimp_image_select_rectangle(image, CHANNEL_OP_REPLACE, 0, 0, width, nheight)
		pdb.gimp_edit_fill(masksw, FILL_FOREGROUND)
		pdb.gimp_edit_fill(maskws, FILL_BACKGROUND)

		pdb.gimp_selection_none(image)

		pdb.gimp_layer_set_visible(original, FALSE)
		gimp.displays_flush()

		return 1


class pyMirImg(gimpplugin.plugin):
	def start(self):
		gimp.main(self.init, self.quit, self.query, self._run)

	def init(self):
		pass

	def quit(self):
		pass

	def query(self):
		authorname = "Me"
		copyrightname = "Me"
		menu_location = "<Image>/Python-Fu/Spiegelbilder"
		date = "now"
		plug_description = "Hilfreich beim erschaffen von Spiegelbildern"
		plug_help = "Mit Hilfe der Schieberegler lassen sich die Bildhälften gegeneinander einstellen." \
					"Der Button Flip spiegelt die Bildhälften, sodass die andere Hälfte genutzt wird." \
					"Reset löscht die erzeugten Ebenen und erstellt sie neu." \
					"Auf Grund des Qualitätsverlustes beim drehen, empfiehlt es sich, das Bild auszurichten," \
					"die eingestellten Werte zu notieren, per reset neu zu starten und die Drehung nur " \
					"einmalig im Zahlenfeld einzugeben und mit Return zu bestätigen." \
					"Die horizontale Bewegung der Bilder ist verlustfrei." \
					"Mit Abbrechen werden die erzeugten Ebenen gelöscht und der Ausgagszustand wieder hergestellt." \
					"Bei Bestätigung mit OK wird das Bild aus dem Original mit der eingestellten Drehung" \
					"neu auf transparentem Hintergrund erzeugt und zugeschnitten." \


		plug_params = [
			(PDB_INT32, "run_mode", "Run mode"),
			(PDB_IMAGE, "image", "Input image"),
			(PDB_DRAWABLE, "drawable", "Input drawable")
			####### 3 params above needed by all scripts using gimpplugin.plugin ######################
			]

		gimp.install_procedure("py_mirror_img",
			plug_description,
			plug_help,
			authorname,
			copyrightname,
			date,
			menu_location,
			"RGB*, GRAY*",
			PLUGIN,
			plug_params,
			[])

	def py_mirror_img(self, runmode, img, drawable):
		mirror_img(runmode, img, drawable)

if __name__ == '__main__':
	pyMirImg().start()
